#pragma once

#include <cstdint>
#include <cstring>
#include <tuple>
#include <vector>

#include "common.hh"
#include "video_decoder.hh"


namespace blky {

std::vector<double> Extract(
    VideoDecoder& dec,
    std::tuple<uint32_t, uint32_t> block_num,
    std::tuple<uint32_t, uint32_t> sensor_num,
    uint32_t utime) {
  std::vector<double> ret;

  blky_extractor_t ex = {};
  ex.block_num_x        = std::get<0>(block_num);
  ex.block_num_y        = std::get<1>(block_num);
  ex.sensor_num_block_x = std::get<0>(sensor_num);
  ex.sensor_num_block_y = std::get<1>(sensor_num);
  ex.samples_per_pix    = 1;
  ex.pix_stride         = 1;
  ex.utime              = utime;
  ex.correl_max_var     = 0.3;
  ex.correl_min_avg     = 0.8;

  blky_extractor_init(&ex);
  while (dec.Decode()) {
    const auto& frame = dec.frame();
    if (frame.iBufferStatus != 1) {
      continue;
    }

    const uint32_t w = static_cast<uint32_t>(frame.UsrData.sSystemBuffer.iWidth);
    const uint32_t h = static_cast<uint32_t>(frame.UsrData.sSystemBuffer.iHeight);

    const uint32_t stride_y  = static_cast<uint32_t>(frame.UsrData.sSystemBuffer.iStride[0]);
    //const uint32_t stride_uv = static_cast<uint32_t>(frame.UsrData.sSystemBuffer.iStride[1]);

    const uint8_t* const* srcp = frame.pDst;

    const double wf = static_cast<double>(w) / static_cast<double>(stride_y);
    const double verts[8] = {
      0, 0,
      0, 1,
      wf, 1,
      wf, 0,
    };
    if (blky_extractor_feed(&ex, srcp[0], stride_y, h, verts)) {
      const size_t psize = ret.size();
      ret.resize(ret.size() + ex.block_num);
      std::memcpy(ret.data() + psize, ex.probs, ex.block_num * sizeof(ret[0]));
    }
  }
  blky_extractor_deinit(&ex);

  return ret;
}

}  // namespace blky
